"use strict";
exports.isArray = function (value) { return Array.isArray(value); };
exports.isFunction = function (value) { return typeof value === 'function'; };
exports.isNil = function (value) { return value === null || typeof value === 'undefined'; };
exports.isNull = function (value) { return value === null; };
exports.isNumber = function (value) { return typeof value === 'number'; };
exports.isObject = function (value) { return typeof value === 'object'; };
exports.isString = function (value) { return typeof value === 'string'; };
exports.isUndefined = function (value) { return typeof value === 'undefined'; };
function toArray(object) {
    return exports.isArray(object) ? object : Object.keys(object).map(function (key) { return object[key]; });
}
exports.toArray = toArray;
exports.getPhone = function (locale) {
    var phones = {
        'en-US': {
            'countryCode': '+1',
            'minLength': 10,
            'pattern': /^(\+?1[-\s]?)?(\(?\d{3}\)?)[-\s]?(\d{3})[-\s]?(\d{4})$/
        },
        'pt-BR': {
            'countryCode': '+55',
            'minLength': 11,
            'pattern': /^(\+?55[-\s]?)?(\([1-9][1-9]\)|[1-9][1-9])[-\s]?(9[1-9]\d{3})[-\s]?(\d{4})$/
        }
    };
    return phones[locale];
};
exports.pad = function (requiredLength, padChar) {
    if (padChar === void 0) { padChar = ' '; }
    return padChar.repeat(requiredLength).slice(0, requiredLength);
};
function equals(a, b) {
    if (a === b) {
        return true;
    }
    if (!exports.isObject(a) || !exports.isObject(b)) {
        return a === b;
    }
    var keysA = Object.keys(a);
    var keysB = Object.keys(b);
    if (keysA.length !== keysB.length) {
        return false;
    }
    // Test for A's keys different from B.
    var hasOwn = Object.prototype.hasOwnProperty;
    for (var _i = 0, keysA_1 = keysA; _i < keysA_1.length; _i++) {
        var key = keysA_1[_i];
        if (!hasOwn.call(b, key) || !equals(a[key], b[key])) {
            return false;
        }
    }
    return true;
}
exports.equals = equals;
function objectContains(partial, object) {
    return Object.keys(partial).every(function (key) { return key in object && object[key] === partial[key]; });
}
exports.objectContains = objectContains;
function deepKeys(obj, stack, parent, intermediate) {
    if (stack === void 0) { stack = []; }
    if (parent === void 0) { parent = null; }
    if (intermediate === void 0) { intermediate = false; }
    Object.keys(obj).forEach(function (el) {
        // Escape . in the element name
        var escaped = el.replace(/\./g, '\\\.');
        // If it's a nested object
        if (exports.isObject(obj[el]) && !exports.isArray(obj[el])) {
            // Concatenate the new parent if exist
            var p = parent ? parent + '.' + el : parent;
            // Push intermediate parent key if flag is true
            if (intermediate)
                stack.push(parent ? p : escaped);
            deepKeys(obj[el], stack, p || escaped, intermediate);
        }
        else {
            // Create and save the key
            var key = parent ? parent + '.' + escaped : escaped;
            stack.push(key);
        }
    });
    return stack;
}
exports.deepKeys = deepKeys;
/**
 * @description
 * return the first n element of an array,
 * if expression provided, is returns as long the expression return truthy
 * @param array
 * @param n {number}
 * @param expression {$parse}
 * @return array or single object
 */
function getFirstMatches(array, n, getter) {
    var count = 0;
    return array.filter(function (elm) {
        var rest = !exports.isUndefined(getter) ? count < n && getter(elm) : count < n;
        count = rest ? count + 1 : count;
        return rest;
    });
}
exports.getFirstMatches = getFirstMatches;
/**
 * @description
 * search for approximate pattern in string
 * @param word
 * @param pattern
 * @returns {*}
 */
function hasApproxPattern(word, pattern) {
    var p = 0;
    for (var i = 0; i <= pattern.length; i++) {
        var index = indexOf(word, p, pattern.charAt(i));
        if (index === -1)
            return false;
        p += index + 1;
    }
    return true;
}
exports.hasApproxPattern = hasApproxPattern;
function indexOf(word, p, c) {
    var j = 0;
    while ((p + j) <= word.length) {
        if (word.charAt(p + j) === c)
            return j;
        j++;
    }
    return -1;
}
exports.convertToDecimal = function (num, decimal) {
    return Math.round(num * Math.pow(10, decimal)) / (Math.pow(10, decimal));
};
//# sourceMappingURL=utils.js.map